package ink.wgink.module.wechat.pay.service.v3.refund.impl;

import com.alibaba.fastjson2.JSONObject;
import ink.wgink.exceptions.ParamsException;
import ink.wgink.exceptions.base.SystemException;
import ink.wgink.module.wechat.pay.enums.PayAuthorizationTypeEnum;
import ink.wgink.module.wechat.pay.enums.PayErrorMsgEnum;
import ink.wgink.module.wechat.pay.pojo.v3.Notice;
import ink.wgink.module.wechat.pay.pojo.v3.refund.Refund;
import ink.wgink.module.wechat.pay.pojo.v3.refund.RefundNoticeCiphertext;
import ink.wgink.module.wechat.pay.pojo.v3.refund.RefundResult;
import ink.wgink.module.wechat.pay.remote.v3.IRefundsRemoteService;
import ink.wgink.module.wechat.pay.service.BasePayService;
import ink.wgink.module.wechat.pay.service.v3.refund.IRefundNoticeService;
import ink.wgink.module.wechat.pay.service.v3.refund.IRefundService;
import ink.wgink.module.wechat.pay.utils.v3.AsyncNoticeResultUtil;
import ink.wgink.properties.wechat.pay.v3.PayProperties;
import ink.wgink.util.map.HashMapUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.RequestMethod;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Map;

/**
 * @ClassName: RefundsServiceImpl
 * @Description: 退款
 * @Author: wanggeng
 * @Date: 2022/11/10 21:29
 * @Version: 1.0
 */
@Service
public class RefundServiceImpl extends BasePayService implements IRefundService {
    @Autowired
    private PayProperties payProperties;
    @Autowired
    private IRefundsRemoteService refundsRemoteService;
    @Autowired
    private IRefundNoticeService refundNoticeService;

    @Override
    public RefundResult refund(String transactionId, Refund refund) throws Exception {
        if (StringUtils.isBlank(transactionId)) {
            throw new ParamsException("微信业务订单ID不能为空");
        }
        if (refund == null) {
            throw new ParamsException("退款内容不能为空");
        }
        Map<String, Object> refundMap = HashMapUtil.beanToMap(refund);
        refundMap.put("transaction_id", transactionId);
        return refund(refundMap);
    }

    @Override
    public RefundResult refundByBusinessOrder(String businessOrderId, Refund refund) throws Exception {
        if (StringUtils.isBlank(businessOrderId)) {
            throw new ParamsException("业务订单ID不能为空");
        }
        if (refund == null) {
            throw new ParamsException("退款内容不能为空");
        }
        Map<String, Object> refundMap = HashMapUtil.beanToMap(refund);
        refundMap.put("out_trade_no", businessOrderId);
        return refund(refundMap);
    }

    @Override
    public RefundResult getByBusinessRefundId(String businessRefundId) throws Exception {
        String urlSuffix = "/v3/refund/domestic/refunds/" + businessRefundId;
        String serialNumber = getSerialNumber();
        String authorization = getAuthorization(RequestMethod.GET, urlSuffix, serialNumber, payProperties, null, PayAuthorizationTypeEnum.WECHATPAY2_SHA256_RSA2048);
        return refundsRemoteService.get(getPayBaseUrl(), authorization, serialNumber, businessRefundId);
    }

    @Override
    public void notice(HttpServletRequest request, HttpServletResponse response, String nonce, String timestamp, String wechatpaySignature) throws IOException {
        try {
            Notice notice = AsyncNoticeResultUtil.getNotice(request, nonce, timestamp, wechatpaySignature);
            String noticeCiphertextJsonString = AsyncNoticeResultUtil.getNoticeCiphertext(notice, payProperties.getApiV3Secretkey());
            RefundNoticeCiphertext refundNoticeCiphertext = JSONObject.parseObject(noticeCiphertextJsonString, RefundNoticeCiphertext.class);
            refundNoticeService.handle(refundNoticeCiphertext);
        } catch (Exception e) {
            LOG.error(e.getMessage(), e);
            if (e instanceof SystemException) {
                AsyncNoticeResultUtil.errorResult(response, PayErrorMsgEnum.SIGN_ERROR_401);
                return;
            }
            AsyncNoticeResultUtil.errorResult(response, PayErrorMsgEnum.ERROR_500);
            return;
        }
        AsyncNoticeResultUtil.successResult(response);
    }

    /**
     * 退款
     *
     * @param refundMap
     * @return
     * @throws Exception
     */
    private RefundResult refund(Map<String, Object> refundMap) throws Exception {
        String urlSuffix = "/v3/refund/domestic/refunds";
        String serialNumber = getSerialNumber();
        JSONObject body = new JSONObject(refundMap);
        String authorization = getAuthorization(RequestMethod.POST, urlSuffix, serialNumber, payProperties, body.toString(), PayAuthorizationTypeEnum.WECHATPAY2_SHA256_RSA2048);
        return refundsRemoteService.refund(getPayBaseUrl(), authorization, serialNumber, body);
    }

}
