package ink.wgink.module.wechat.pay.utils.v3;

import com.alibaba.fastjson.JSONObject;
import ink.wgink.exceptions.ParamsException;
import ink.wgink.exceptions.base.SystemException;
import ink.wgink.module.wechat.pay.enums.PayErrorMsgEnum;
import ink.wgink.module.wechat.pay.manager.v3.PayManager;
import ink.wgink.module.wechat.pay.pojo.v3.Notice;
import ink.wgink.module.wechat.pay.pojo.v3.PayErrorResponse;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.nio.charset.StandardCharsets;
import java.security.GeneralSecurityException;

/**
 * @ClassName: AsyncNoticeResultUtil
 * @Description: 异步通知返回工具
 * @Author: wanggeng
 * @Date: 2022/11/11 23:27
 * @Version: 1.0
 */
public class AsyncNoticeResultUtil {

    private static final Logger LOG = LoggerFactory.getLogger(AsyncNoticeResultUtil.class);

    /**
     * 获取通知内容
     *
     * @param request
     * @return
     * @throws IOException
     */
    public static Notice getNotice(HttpServletRequest request, String nonce, String timestamp, String wechatpaySignature) throws Exception {
        String payNoticeJsonString = AsyncNoticeResultUtil.getBodyString(request);
        if (StringUtils.isBlank(payNoticeJsonString)) {
            throw new ParamsException("请求body为空");
        }
        LOG.debug("NoticeJsonString: {}", payNoticeJsonString);
        Notice notice = JSONObject.parseObject(payNoticeJsonString, Notice.class);
        LOG.debug("wechatpaySignature: {}", wechatpaySignature);
        LOG.debug("验证签名");
        boolean checkVerify = PayVerifyUtil.verifySignature(wechatpaySignature, payNoticeJsonString, nonce, timestamp, PayManager.getInstance().getPlatformCertificate());
        if (!checkVerify) {
            throw new SystemException("签名验证失败");
        }
        return notice;
    }

    /**
     * 获取通知密文
     *
     * @param notice
     * @param apiV3Secretkey
     * @return
     * @throws GeneralSecurityException
     * @throws IOException
     */
    public static String getNoticeCiphertext(Notice notice, String apiV3Secretkey) throws GeneralSecurityException, IOException {
        LOG.debug("解密内容");
        Notice.Resource resource = notice.getResource();
        PayAesUtil aesUtil = new PayAesUtil(apiV3Secretkey.getBytes(StandardCharsets.UTF_8));
        return aesUtil.decryptToString(resource.getAssociated_data().getBytes(StandardCharsets.UTF_8), resource.getNonce().getBytes(StandardCharsets.UTF_8), resource.getCiphertext());
    }

    /**
     * 错误结果
     *
     * @param payErrorMsgEnum
     */
    public static void errorResult(HttpServletResponse response, PayErrorMsgEnum payErrorMsgEnum) {
        PayErrorResponse payErrorResponse = new PayErrorResponse();
        payErrorResponse.setCode(payErrorMsgEnum.getErrorCode());
        payErrorResponse.setMessage(payErrorMsgEnum.getSummary());
        response.setContentType("application/json; charset=UTF-8");
        response.setStatus(payErrorMsgEnum.getCode());
        try (PrintWriter writer = response.getWriter()) {
            writer.println(JSONObject.toJSONString(payErrorResponse));
            writer.flush();
        } catch (IOException e) {
            LOG.error(e.getMessage(), e);
        }
    }

    /**
     * 成功结果
     */
    public static void successResult(HttpServletResponse response) {
        response.setContentType("application/json; charset=UTF-8");
        response.setStatus(HttpStatus.OK.value());
        JSONObject successJsonObject = new JSONObject();
        successJsonObject.put("code", "SUCCESS");
        successJsonObject.put("message", "成功");
        try (PrintWriter writer = response.getWriter()) {
            writer.println(successJsonObject.toJSONString());
            writer.flush();
        } catch (IOException e) {
            LOG.error(e.getMessage(), e);
        }
    }

    public static String getBodyString(HttpServletRequest request) throws IOException {
        BufferedReader reader = request.getReader();
        StringBuilder bodyString = new StringBuilder();
        for (String line; (line = reader.readLine()) != null; ) {
            bodyString.append(line);
        }
        return bodyString.toString();
    }

}
